import minio
import io
import os
import sys

from qwertyui import urlparse


def get_minio_client(url, access_key, secret_key, region='eu-central-1', bucket=None):
    """
    Wrapper for getting minio.Minio class.
    Can optionally generate a bucket if parameter is provided.
    """

    parsed = urlparse(url)
    secure = parsed.scheme == 'https'

    mc = minio.Minio(
        parsed.netloc,
        access_key,
        secret_key,
        region=region,
        secure=secure
    )

    if not bucket:
        return mc

    if not mc.bucket_exists(bucket):
        mc.make_bucket(bucket, location=region)

    return mc


def upload_file(client, bucket, file_path, minio_directory, content_type=None, metadata=None):
    size = os.stat(file_path).st_size
    file_name = os.path.split(file_path)[1]
    minio_file_path = '/'.join(minio_directory, file_name)

    with io.open(file_path, 'rb') as f:
        client.put_object(
            bucket,
            minio_file_path,
            f,
            size,
            content_type=content_type or 'application/octet-stream',
            metadata=metadata
        )


def size(client, bucket, file_path):
    return client.stat_object(bucket, file_path).size



import datetime
import os
import requests
import tempfile

from qwertyui import urlparse


def download_backup(host, db, master_pwd, backup_dir=None, backup_format='zip', chunk_size=16384):
    """
    Downloads a full backup of an ODOO database and all data files.
    """

    if not backup_dir:
        backup_dir = tempfile.mkdtemp()

    if not os.path.exists(backup_dir):
        os.makedirs(backup_dir)

    session = requests.Session()

    session.get(
        '%s/web' % host,
        params={'db': db},
        verify=True
    )
    resp = session.post(
        '%s/web/database/backup' % host,
        {'master_pwd': master_pwd, 'name': db, 'backup_format': backup_format},
        stream=True,
        verify=True
    )
    if 'application/octet-stream' not in resp.headers['Content-Type']:
        raise Exception('Content-type is not application/octet-stream, no zipfile received!')

    now = datetime.datetime.now()
    parsed = urlparse(host)
    file_name = 'backup-%s-%s-%s.%s' % (
        parsed.hostname,
        db,
        now.strftime('%Y%m%d-%H%M%S'),
        backup_format
    )
    file_path = os.path.join(backup_dir, file_name)

    with open(file_path, 'wb') as f:
        size = 0
        for chunk in resp.iter_content(chunk_size):
            f.write(chunk)
            size += len(chunk)

    return file_path, size



import datetime
import math
import re


class PeriodicBackupRemover:
    """
    Use this to remove backups based on intervals. For example you can have the following rules:

    backup_rules = [
        {
            'upto': (3, 'day'),
            'interval': (2, 'hour'),
        },
        {
            'upto': (1, 'month'),
            'interval': (1, 'day'),
        },
        {
            'upto': (1, 'year'),
            'interval': (1, 'week'),
        },
    ]

    This means that: backups up to 3 days are store with intervals at least 2 hours,
    backups older than 3 days but more recent than 1 month ago are stored in at least
    1 day intervals while backups older than 1 month and more recent than 1 year are
    stored in at least 1 week intervals. Backups older than 1 year will be discarded.

    And then you can initialize remover like this:

    remover = PeriodicBackupRemover()
    remover.add_rules(backup_rules)

    Remover also accepts a file name format.

    By default backups are in the format:

    backup-my.host.name-my-db-name-20170629-120000.zip

    We need both host name and db name to differentiate between various hosts and their dbs.
    Each (host, db) pair is treated as 1 group of backups to operate on.
    """

    def __init__(self,
                 backup_file_name_re=r'backup-(.*)-(\d{4})(\d{2})(\d{2})-(\d{2})(\d{2})(\d{2}).zip',
                 host_db_re=r'(.+\..+\.\w+)-(.+)'):
        self.rules = []
        self.backup_file_name_re = re.compile(backup_file_name_re)
        self.host_db_re = re.compile(host_db_re)

    @staticmethod
    def parse_date_format(num, unit):
        seconds_in_hour = 60*60
        seconds_in_day = seconds_in_hour*24

        if unit == 'hour':
            return num*seconds_in_hour
        elif unit == 'day':
            return num*seconds_in_day
        elif unit == 'week':
            return num*seconds_in_day*7
        elif unit == 'month':
            return num*seconds_in_day*30
        elif unit == 'year':
            return num*seconds_in_day*365

        raise Exception('Unknown upto unit %s' % unit)

    @staticmethod
    def rule_satisfied(rule, dates, date):
        if not PeriodicBackupRemover.check_rule_upto(rule, dates, date):
            return False

        if not PeriodicBackupRemover.check_rule_interval(rule, dates, date):
            return False

        return True

    @staticmethod
    def check_rule_upto(rule, dates, date):
        now = datetime.datetime.now()

        upto_num, upto_unit = rule['upto']
        upto_sec = PeriodicBackupRemover.parse_date_format(upto_num, upto_unit)

        diff = now - date
        if diff.total_seconds() > upto_sec:
            return False

        return True

    @staticmethod
    def check_rule_interval(rule, dates, date):
        # OK, diff in range, check if interval is satisfied
        interval_num, interval_unit = rule['interval']
        interval_sec = PeriodicBackupRemover.parse_date_format(interval_num, interval_unit)

        # Empty list
        if not dates:
            return True
        if len(dates) == 1:
            diff = dates[0] - date
            return abs(diff.total_seconds()) >= interval_sec

        dates = sorted(dates)

        # Find position in sorted dates list and check if 'date'
        # can be fitted between them
        pos = -1
        for i, d in enumerate(dates):
            if date <= d:
                pos = i
                break

        if pos > 0:
            d = dates[pos - 1]
            diff = date - d
            if abs(diff.total_seconds()) < interval_sec:
                return False

        if pos < len(dates) - 1:
            d = dates[pos]
            diff = date - d
            if abs(diff.total_seconds()) < interval_sec:
                return False

        return True

    def add_rules(self, rules):
        for rule in rules:
            if isinstance(rule, dict):
                self.add_rule(
                    rule['upto'],
                    rule['interval']
                )
            else:
                self.add_rule(
                    rule[0],
                    rule[1]
                )

    def add_rule(self, upto, interval):
        self.rules.append({
            'upto': upto,
            'interval': interval,
        })

    def parse_backup_file_name(self, file_name):
        try:
            match = self.backup_file_name_re.match(file_name).groups()
        except AttributeError:
            return

        host_db = match[0]
        host, db = self.host_db_re.match(host_db).groups()

        return {
            'date': datetime.datetime(*map(int, match[1:])),
            'db': db,
            'host': host,
            'file_name': file_name,
        }

    def keep_backup(self, kept_backups, backup):
        backup_dates = [b['date'] for b in kept_backups]

        for rule in self.rules:
            if self.rule_satisfied(rule, backup_dates, backup['date']):
                return True

        # No rule was satisfied -- don't keep the backup
        return False

    def filter_backups(self, file_names):
        backups = {}
        remove_backups = {}
        unknown_file_names = []

        for file_name in file_names:
            backup = self.parse_backup_file_name(file_name)
            if backup is None:
                unknown_file_names.append(file_name)
                continue
            key = (backup['host'], backup['db'])
            backups.setdefault(key, [])
            remove_backups.setdefault(key, [])

            if self.keep_backup(backups[key], backup):
                backups[key].append(backup)
            else:
                remove_backups[key].append(backup)

        return {
            'keep': backups,
            'remove': remove_backups,
            'unknown': unknown_file_names,
        }



#!/usr/bin/env python

from setuptools import find_packages, setup


setup(name='qwertyui',
      version='1.0.3',
      description='Some common Python functions and algorithms',
      author='Przemyslaw Kaminski',
      author_email='cgenie@gmail.com',
      url='https://github.com/CGenie/qwertyui',
      packages=find_packages(exclude=['tests.py']),
      install_requires=[
          'minio==2.2.3',
          'requests==2.18.1',
      ]
)



import datetime
import unittest


class PeriodicBackupRemoverTest(unittest.TestCase):
    def setUp(self):
        from .periodic_backup_remover import PeriodicBackupRemover

        BACKUP_RULES = [
            {
                'upto': (3, 'day'),
                'interval': (2, 'hour'),
            },
            {
                'upto': (1, 'month'),
                'interval': (1, 'day'),
            },
            {
                'upto': (1, 'year'),
                'interval': (1, 'week'),
            },
        ]

        self.remover = PeriodicBackupRemover()

        for rule in BACKUP_RULES:
            self.remover.add_rule(rule['upto'], rule['interval'])

    def test_rules(self):
        file_names = [
            'backup-host1.example.com-db1-20170114-060653.zip',
            'backup-host1.example.com-db1-20170115-064242.zip',
            'backup-host1.example.com-db1-20170116-072720.zip',
            'backup-host1.example.com-db1-20170116-140950.zip',
            'backup-host1.example.com-db1-20170117-144503.zip',
            'backup-host1.example.com-db1-20170118-150440.zip',
            'backup-host1.example.com-db1-20170119-152427.zip',
            'backup-host1.example.com-db1-20170120-154418.zip',
            'backup-host1.example.com-db1-20170121-160328.zip',
            'backup-host1.example.com-db1-20170122-162216.zip',
            'backup-host1.example.com-db1-20170123-164124.zip',
            'backup-host1.example.com-db1-20170124-170108.zip',
            'backup-host2.example.com-db1-20170114-060655.zip',
            'backup-host1.example.com-db2-20170201-115109.zip',
            'backup-host1.example.com-db2-20170201-205858.zip',
            'backup-host1.example.com-db1-20170122-162216.zip',
            'backup-host1.example.com-db2-20170202-171612.zip',
            'backup-host1.example.com-db2-20170202-050529.zip',
            'backup-host1.example.com-db2-20170130-055307.zip',
            'backup-host1.example.com-db2-20170131-213921.zip',
        ]

        for file_name in file_names:
            self.assertIsNotNone(self.remover.parse_backup_file_name(file_name), "File name %s cannot be parsed" % file_name)

        self.assertEqual(
            self.remover.parse_backup_file_name(file_names[0]),
            {
                'date': datetime.datetime(2017, 1, 14, 6, 6, 53),
                'db': 'db1',
                'host': 'host1.example.com',
                'file_name': file_names[0],
            }
        )

        now = datetime.datetime.now()

        self.assertFalse(
            self.remover.rule_satisfied(
            {
                'upto': (3, 'day'),
                'interval': (2, 'hour'),
            },
            [],
            now - datetime.timedelta(days=4),
            )
        )

        self.assertTrue(
            self.remover.rule_satisfied(
                {
                    'upto': (3, 'day'),
                    'interval': (2, 'hour'),
                },
                [],
                now - datetime.timedelta(days=2),
            )
        )

        self.assertTrue(
            self.remover.rule_satisfied(
                {
                    'upto': (3, 'day'),
                    'interval': (2, 'hour'),
                },
                [
                    now - datetime.timedelta(days=2, hours=3),
                ],
                now - datetime.timedelta(days=2),
            )
        )

        self.assertFalse(
            self.remover.rule_satisfied(
                {
                    'upto': (3, 'day'),
                    'interval': (2, 'hour'),
                },
                [
                    now - datetime.timedelta(days=2, hours=1),
                ],
                now - datetime.timedelta(days=2),
            )
        )

        self.assertTrue(
            self.remover.rule_satisfied(
                {
                    'upto': (3, 'day'),
                    'interval': (2, 'hour'),
                },
                [
                    now - datetime.timedelta(days=1, hours=21),
                ],
                now - datetime.timedelta(days=2),
            )
        )

        self.assertFalse(
            self.remover.rule_satisfied(
                {
                    'upto': (3, 'day'),
                    'interval': (2, 'hour'),
                },
                [
                    now - datetime.timedelta(days=1, hours=23),
                ],
                now - datetime.timedelta(days=2),
            )
        )

        self.assertTrue(
            self.remover.check_rule_interval(
                {
                    'upto': (3, 'day'),
                    'interval': (2, 'hour'),
                },
                [
                    now - datetime.timedelta(days=1, hours=21),
                    now - datetime.timedelta(days=1, hours=19),
                ],
                now - datetime.timedelta(days=2),
            )
        )
